﻿'Copyright © exc-jdbi 2020
'www.vb-paradise.de/index.php/User/23082-exc-jdbi/
'© PwKeySetHolder 2020 V1

'Siehe beiligende Lizenz



Option Strict On
Option Explicit On
Imports System.Security.Cryptography
Imports exc.jdbi.Cryptographie.PwKeySetHolders

'HINWEIS:
'Unter Verweis "System.Security" einbinden.
'Unter Verweis "PwKeySetHolder" einbinden.


'Hinweis
'Nur für Windowsplattformen
'https://docs.microsoft.com/de-de/dotnet/api/system.security.cryptography.protecteddata?view=netframework-4.7.1#remarks

Public Module PwKeySetTest
  Friend ReadOnly Rand As New Random
  Public Sub Main()
    Dim cnt = 10

    Dim sw = Stopwatch.StartNew

    TestSingle(cnt)
    Console.WriteLine($"{NameOf(TestSingle)} t = {sw.ElapsedMilliseconds}ms")
    Console.WriteLine()

    sw.Restart()
    TestAllSamples(cnt)
    Console.WriteLine($"{NameOf(TestAllSamples)} t = {sw.ElapsedMilliseconds}ms")
    Console.WriteLine()

    'Daten zum Senden über das Internet
    TestDispatchData()

    'Speichern auf die HDD
    TestPasswordInputBox()

    'Versenden übers Internet
    TestDiffieHellman()

    Console.WriteLine()
    Console.WriteLine()
    Console.WriteLine("FINISH")

    Console.ReadLine()
  End Sub

  Private Sub TestSingle(cnt As Int32)
    Console.WriteLine($"{NameOf(TestSingle)} cnt = {cnt}")

    Dim protection As DataProtectionScope
    For i = 0 To cnt - 1
      protection = If((Rand.Next And 1) = 0,
                       DataProtectionScope.CurrentUser,
                       DataProtectionScope.LocalMachine)

      'KeySet erstellen und wieder aus
      'PwKeySetHolder extrahieren.
      Using pks = New PwKeySetHolder
        pks.CreateNewKeySet(300, 100, protection)
        Dim ks1 = pks.ToPwKeySet
        Using t = pks.ToPwKeySetAsync
          Dim ks2 = t.Result ': t.Wait()
          Debug.Assert(Equality(ks1, ks2))
        End Using
      End Using
    Next
  End Sub
  Private Sub TestAllSamples(cnt As Int32)
    If cnt <= 0 Then Return

    Dim sw = Stopwatch.StartNew

    'KeySet erstellen und wieder aus
    'PwKeySetHolder extrahieren.
    Dim pks1 = New PwKeySetHolder
    pks1.CreateNewKeySet(10, 50, 100)
    Dim _ks1 = pks1.ToPwKeySet
    Dim _ks2 = pks1.ToPwKeySet
    Debug.Assert(Equality(_ks1, _ks2))
    pks1.Dispose()
    Console.WriteLine($"Without Using t = {sw.ElapsedMilliseconds}ms")

    sw.Restart()

    'KeySet erstellen und wieder aus
    'PwKeySetHolder extrahieren.
    'Hier mal die Async-Variante
    Using pks = New PwKeySetHolder
      Using t1 = pks.CreateNewKeySetAsync(7, 50)
        t1.Wait()
        Using t2 = pks.ToPwKeySetAsync, t3 = pks.ToPwKeySetAsync
          Debug.Assert(Equality(t2.Result, t3.Result))

          'Beide Task müssen komplett abgeschlossen sein, 
          'bevor disposed werden kann, ansonsten gibt es 
          'unter Umständen ein Exeption
          t2.Wait() : t3.Wait()
        End Using
      End Using
    End Using

    Console.WriteLine($"With Using t = {sw.ElapsedMilliseconds}ms")

    'Mit ToNewPwKeySetAsync(), die einen Task zurückgibt,
    'der überall hin durchgereicht, und vor Gebrauch mit 
    'Result() ausgeführt werden kann.
    't1.Wait() wird hier nicht zwingend gebraucht, da 
    'nachher noch ToPwKeySet() ausgeführt wird.
    sw.Restart()
    Using pks = New PwKeySetHolder
      Using t1 = pks.ToNewPwKeySetAsync(75, 50, 1000)
        Dim k1 = t1.Result ':t1.Wait()
        Dim k2 = pks.ToPwKeySet
        Debug.Assert(Equality(k1, k2))
      End Using
    End Using
    Console.WriteLine($"NPSAsync t = {sw.ElapsedMilliseconds}ms")

    sw.Restart()

    'Eigenes KeySet einlesen
    Dim selfdata = RngJagged(10, 20, 50)
    Using pks = New PwKeySetHolder
      'Eigene Schlüssel einlesen
      pks.ImportKeySet(selfdata)
      Dim ks1 = pks.ToPwKeySet
      Dim ks2 = pks.ToPwKeySet
      Debug.Assert(Equality(ks1, ks2))
      Debug.Assert(Equality(selfdata, ks2))
    End Using
    Console.WriteLine($"SelfKeySet t = {sw.ElapsedMilliseconds}ms")

    sw.Restart()
    'Versendedaten erstellen und wieder einlesen
    selfdata = RngJagged(10, 20, 50)
    Using pks = New PwKeySetHolder
      pks.ImportKeySet(selfdata, DataProtectionScope.LocalMachine)
      Dim ks1 = pks.ToPwKeySet
      Dim ks2 = pks.ToPwKeySet
      Debug.Assert(Equality(ks1, ks2))
      Debug.Assert(Equality(selfdata, ks1))

      Using pks2 = New PwKeySetHolder
        'Versendedaten erstellen mit Using(!)

        'WICHTIG: 
        'Das Passwort ist Plaintext und muss vor dem versenden
        'übers Internet verkryptet werden.
        'Die Chipherdata ist verkryptet mit dem beiliegendem Passwort.
        Using dispatch = pks.ToDispatchData
          'Versendedaten neu einlesen.
          pks2.ImportDispatchData(dispatch)
          Dim ks3 = pks.ToPwKeySet
          Dim ks4 = pks.ToPwKeySet
          Debug.Assert(Equality(ks3, ks4))
          Debug.Assert(Equality(ks1, ks3))
          Debug.Assert(Equality(selfdata, ks3))
        End Using
      End Using
    End Using

    Console.WriteLine($"2 x DispatchData t = {sw.ElapsedMilliseconds}ms")
    Console.WriteLine()

    TestAllSamples(cnt - 1)
  End Sub
  Private Sub TestDispatchData()
    '"DispatchData" hat IDisposable implementiert.
    'Daher am Besten immer gleich mit dem Using-Block
    'arbeiten, dann werden die Daten auch gleich korrekt
    'wieder zurückgesetzt.

    Using pks = New PwKeySetHolder
      pks.CreateNewKeySet(2, 50)
      Dim ks1 = pks.ToPwKeySet
      Dim ks2 = pks.ToPwKeySet
      Debug.Assert(Equality(ks1, ks2))

      Using pks2 = New PwKeySetHolder
        'Versendedaten erstellen mit Using(!)
        Using dispatch = pks.ToDispatchData
          pks2.ImportDispatchData(dispatch)
          Dim ks3 = pks.ToPwKeySet
          Dim ks4 = pks.ToPwKeySet
          Debug.Assert(Equality(ks3, ks4))
          Debug.Assert(Equality(ks1, ks4))
        End Using
      End Using
    End Using

    Dim selfdata = RngJagged(10, 20, 50)
    Using pks = New PwKeySetHolder
      pks.ImportKeySet(selfdata, DataProtectionScope.LocalMachine)
      Dim ks1 = pks.ToPwKeySet
      Dim ks2 = pks.ToPwKeySet
      Debug.Assert(Equality(ks1, ks2))
      Debug.Assert(Equality(selfdata, ks1))

      Using pks2 = New PwKeySetHolder
        'Versendedaten erstellen mit Using(!)
        Using dispatch = pks.ToDispatchData
          pks2.ImportDispatchData(dispatch)
          Dim ks3 = pks.ToPwKeySet
          Dim ks4 = pks.ToPwKeySet
          Debug.Assert(Equality(ks3, ks4))
          Debug.Assert(Equality(ks1, ks3))
          Debug.Assert(Equality(selfdata, ks3))
        End Using
      End Using
    End Using
  End Sub
  Private Sub TestPasswordInputBox()
    'Dim selfdata = RngJagged(10, 20, 50)
    Dim selfdata = FixKeySet()
    Dim protection As DataProtectionScope
    protection = If((Rand.Next And 1) = 0,
                       DataProtectionScope.CurrentUser,
                       DataProtectionScope.LocalMachine)
    Using pks = New PwKeySetHolder
      pks.ImportKeySet(selfdata, protection)
      pks.SavePwKeySet()
    End Using

    'Wer's wissen will kann am nächsten Tag den Computer
    'neu Aufstarten und ab hier den Code nochmals laufen
    'lassen. Eventuell auch in einem anderen Benutzerkonto.

    Using pks = New PwKeySetHolder
      pks.LoadPwKeySet()
      If pks.Busy Then
        Dim ks1 = pks.ToPwKeySet
        Dim ks2 = pks.ToPwKeySet
        Debug.Assert(Equality(ks1, ks2))
        Debug.Assert(Equality(selfdata, ks2))
      End If
    End Using
  End Sub
  Private Sub TestDiffieHellman()
    'Ideal zum Versenden von Daten übers Internet
    MainTestDH()
  End Sub


  Private Function RngJagged(size As Int32, keysize As Int32) As Byte()()
    Dim result = New Byte(size - 1)() {}
    For i = 0 To result.Length - 1
      Dim tmp = New Byte(keysize - 1) {}
      Rand.NextBytes(tmp)
      result(i) = tmp
    Next
    Return result
  End Function

  Private Function RngJagged(size As Int32, keysizemin As Int32, keysizemax As Int32) As Byte()()
    Dim result = New Byte(size - 1)() {}
    For i = 0 To result.Length - 1
      Dim sz = Rand.Next(keysizemin, keysizemax + 1)
      Dim tmp = New Byte(sz - 1) {}
      Rand.NextBytes(tmp)
      result(i) = tmp
    Next
    Return result
  End Function
  Private Function Equality(b1() As Byte, b2() As Byte) As Boolean
    Return b1.SequenceEqual(b2)
  End Function
  Private Function Equality(b1()() As Byte, b2()() As Byte) As Boolean
    If Not b1.Length = b2.Length Then Return False
    For i = 0 To b1.Length - 1
      If Not Equality(b1(i), b2(i)) Then
        Return False
      End If
    Next
    Return True
  End Function
  Private Function FixKeySet() As Byte()()
    'Eine Jagged nach Wahl, für das Testen.
    Return New Byte()() _
      {
        (New Byte() {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}),
        (New Byte() {9, 8, 7, 6, 5, 4, 3, 2, 1, 0}),
        (New Byte() {0, 1, 5, 9, 7, 3, 2, 4, 8, 6}),
        (New Byte() {6, 8, 4, 2, 3, 7, 9, 5, 1, 0})
      }
  End Function
End Module

